#import <Foundation/Foundation.h>

/*
 Use this when:
 
 You want to pass an input stream to something that will read from it (eg. doing a file upload).
 and
 You want to produce the data dynamically, rather than first sticking it all in an NSData or a temp file.
 
 Loosely adapted from some of PostController in SimpleURLConnections sample code.
 */
@class BoundStreamWriter;

@interface BoundStreamWriter : NSObject <NSStreamDelegate> {
	NSInputStream * _instream;
	NSOutputStream * _outstream;
	NSData * _buffer;
	int  _bufferPos;
	NSDataReturningBlock  _fillBuffer;
	BasicBlock  _onSuccess;
	StringArgBlock  _onFailure;
	NSMutableData *all;
}
@property (retain) NSInputStream *instream;
@property (retain) NSOutputStream *outstream;
@property (retain) NSData *buffer;
@property (assign) int bufferPos;

@property (readonly) int bufferSize;
@property (copy) BasicBlock onSuccess;
@property (copy) StringArgBlock onFailure;
@property (copy) NSDataReturningBlock fillBuffer;

+ (BoundStreamWriter*) boundStreamWriter;
- (void) start;

@end

