#!/bin/sh

exec 2>&1

echo "---------------------- Beginning Installation ---------------------------"
date
successFile=$1
automatic=$2
echo "auto: $automatic"

set errexit
set -x

# this isn't set when we're running from the dummy transitional package because of the "su - ..."
export PATH=/usr/bin:/bin:/usr/sbin:/sbin
username=$USER


# ------------------- The following section is shared with the uninstaller. Keep them the same. ----------------------

logDir=/Library/Logs/FarSuite
dir="/Library/FarSuite"
olddir="/Library/Application Support/FarSuite"
appId=com.flyingmac.farsuite
daemonId=com.flyingmac.farsuite.daemon
mainLog=$logDir/farsuite.log
filesDir=$dir/files
mainPrefs=$filesDir/main.plist
daemonPrefs=$filesDir/daemon.plist

function log {
	echo $* >&2
}

function killprocs() {
	targ=$1
	name=$2
	
	pids=`ps ax | fgrep "$targ" | grep -v grep | cut -c1-6`
	if [ ! -z "$pids" ]; then
		log "sending QUIT $name $pids"
		kill -QUIT $pids
		log "Waiting 5 seconds..."
		sleep 5
	fi
	pids=`ps ax | fgrep "$targ" | grep -v grep | cut -c1-6`
	if [ ! -z "$pids" ]; then
		log "sending HUP $name $pids"
		kill -HUP $pids
		log "Waiting 5 seconds..."
		sleep 5
	fi
	pids=`ps ax | fgrep "$targ" | grep -v grep | cut -c1-6`
	if [ ! -z "$pids" ]; then
		log "sending KILL $name $pids"
		kill -KILL $pids
		log "Waiting 5 seconds..."
		sleep 5
	fi
}

# Note this also needs to work when in the old location -  /Library/Application Support. 
# It does, as there's no dependency on installed location.
function stopEverything {
	# stop application
	appLaunchd=/Library/LaunchAgents/$appId.plist
	if [ -f "$appLaunchd" ]; then
		status "Unloading application"
		sudo -u "$username" launchctl unload -S Aqua "$appLaunchd" &
		sleep 5
		kill %% || true
	else
		log "The application isn't in launchd"
	fi
	status "Checking for stray application"
	killprocs 'FarSuite.app/Contents/MacOS/FarSuite' Application


	# stop daemon
	if [ -f /Library/LaunchDaemons/$daemonId.plist ]; then
		status "Unloading daemon"
		launchctl unload /Library/LaunchDaemons/$daemonId.plist &
		sleep 5
		kill %% || true
	fi
	status "Checking for stray daemon"
	killprocs 'FarSuiteDaemon.app/Contents/MacOS/FarSuiteDaemon' Daemon
	
	# stop stray agents
	status "Checking for stray agents"
	killprocs "FarFinderAgent.app/Contents/MacOS/FarFinderAgent" FarFinderAgent
	killprocs "WebjimboAgent.app/Contents/MacOS/WebjimboAgent" WebjimboAgent
}

# ------------------- End uninstaller shared section. ----------------------


function status {
	log $*
	./nsnotifier com.flyingmac.installer.status "$*"
}

mkdir -p "$logDir"
umask 0022
here=${0%/*} # parent folder
[[ -z $here ]] && exit 1
cd "$here"

unset errexit
up=${here%/*}  # up another dir
version=$(defaults read "$up/Info" CFBundleShortVersionString)
log "-------- version $version ------"
set errexit

# -------------------------------------- Preflight ------------------------------------

log "Username is $username"

if [ -e "$dir" -o -e "$olddir" ]; then
	ffinfo="$dir/Modules/FarFinder.bundle/Contents/Info"
	if [ -e "$ffinfo.plist" ]; then
		unset errexit
		oldffversion=$(defaults read "$ffinfo" CFBundleShortVersionString)
		if [[ ${oldffversion:0:2} = "1." || ${oldffversion:0:6} = "beta2." ]]; then
			log "This is an upgrade from FF v1 or beta"
			defaults write "/Library/Preferences/$appId" ffV1wasInstalled -bool YES
		fi
		set errexit
	fi	
	log "This is an upgrade"
	stopEverything
else
	log "This is a new installation"
fi

# Remove the old links we used to use
[ -L /Applications/FarSuite ] && rm -f /Applications/FarSuite
[ -L /Applications/FarFinder ] && rm -f /Applications/FarFinder
[ -L /Applications/Webjimbo ] && rm -f /Applications/Webjimbo

# no longer used
rm /Users/*/Library/Keychains/FarFinder.keychain

# Remove the FarFinder app (it'll be reinstalled if we're still called FarFinder)
[ -e /Applications/FarFinder.app ] && rm -rf /Applications/FarFinder.app

# Move from /Library/Application Support to /Library. Most will just be overwritten, but it takes care of Licences and Files
if [ -e "$olddir" ]; then
	if [ ! -e "$dir" ]; then
		mv "$olddir" "$dir"
		
		# Remove keychain as the relocated agent won't have permission to read it
		rm /Users/*/Library/Keychains/FarFinder.keychain || true
	else
		# Unexpected. Oh well, remove the old one.
		rm -rf "$olddir"
	fi
fi

# Remove FarFinderServices - we don't use it any more
if [ -e /Library/Services/FarFinderServices.app ]; then
	unset errexit
	killall FarFinderServices
	set errexit
	rm -rf /Library/Services/FarFinderServices.app
fi


# -------------------------------------- Install ------------------------------------

function inst {
	set -e
	[ ! -d "$2" ] && mkdir -p "$2"  # eg. /Library/LaunchAgents might not exist
	name=${1##*/}  # like basename
	dest="$2/$name"
	rm -rf "$dest"
	ditto "$1" "$dest"
	chown -R root:admin "$dest"
	chmod u+rwX "$dest"
	chmod go-w "$dest"
}

function makedir {
	set -e
	[ -e "$1" -a ! -d "$1" ] && { log "Exists but not a directory: $1"; exit 1; }
	[ ! -d "$1" ] && mkdir -p "$1"
	chmod 755 "$1"
}

function relocate {
	oldfile="$1"
	newfile="$2"
	if [ -e "$oldfile" ]; then
		if [ -e "$newfile" ]; then
			rm -f "$oldfile"
		else
			mv "$oldfile" "$newfile"
		fi
	fi
}


status "Installing files"
makedir "$dir"

for f in Main/*; do
	inst "$f" "$dir"
done
chmod 4755 "$dir/FarSuiteAgentLauncher"
inst "$daemonId.plist" /Library/LaunchDaemons
inst "$appId.plist" /Library/LaunchAgents
makedir "$filesDir"
# Set (or fix) permissions on the whole files dir, including ACLs. Had someone with erroneous ACLs on settings files causing a problem.
chmod -R a+rwX "$filesDir"
chmod -RN "$filesDir" # remove ACLs

#prefs relocation: v2.7
relocate /Library/Preferences/com.flyingmac.farsuite.plist $mainPrefs
relocate /Library/Preferences/com.flyingmac.farsuite.daemon.plist $daemonPrefs

# licences folder
supportDir="/Library/Application Support/FlyingMac"
licencesDir="$supportDir/Licences"
oldLicencesDir=$dir/Licences
makedir "$supportDir" # separately, so the permissions get set on it
makedir "$licencesDir"
chmod 777 "$licencesDir"
if [ -d "$oldLicencesDir" ]; then
	for f in "$oldLicencesDir"/*; do
		[ -f "$f" ] && mv "$f" "$licencesDir"  # get blah/* if folder empty, hence the test
	done
	rm -rf "$oldLicencesDir"
fi

modulesDir="$dir/Modules"
makedir "$modulesDir"

function installModule {
	set -e
	moduleDir=$1
	moduleName=${moduleDir##*/}  # like basename
	
	inst "$moduleDir"/*.bundle  "$modulesDir"
	inst "$moduleDir"/application/*.app /Applications
	
	# Install the CMM if the module has one, and if this is 10.5
	osversion=$(sw_vers -productVersion)
	if [[ ${osversion:0:4} = 10.5 ]]; then
		cmmDir="$moduleDir/CMM"
		if [ -d "$cmmDir" ]; then
			for cmm in "$cmmDir"/*.bundle; do
				inst "$cmm" "/Library/Contextual Menu Items"
			done
		# No CMM in package - uninstall if present (this will handle downgrading)
		else
			if [ -d "/Library/Contextual Menu Items" ]; then
				find "/Library/Contextual Menu Items" -name $moduleName\* -print | while read cmm; do
					rm -rf "$cmm"
				done
			fi
		fi
	fi
	
	# Install the service if the module has one
	serviceDir="$moduleDir/Service"
	if [ -d "$serviceDir" ]; then
		for service in "$serviceDir"/*.service; do
			# kill the service if it's running
			withoutDot=${service%.*}
			killprocs $withoutDot
			
			inst "$service" "/Library/Services"
		done
	# Otherwise, remove a service if it's there. Useful when downgrading.
	else
		find /Library/Services -name $moduleName\* -print | while read service; do
			# kill the service if it's running
			withoutDot=${service%.*}
			killprocs $withoutDot
			
			rm -rf "$service"
		done
	fi
}

# Manual install
if [ -z "$automatic" ]; then
	for f in Modules/*; do
		installModule "$f"
	done

# Automatic (ie. self-update) - modules in neighbouring packages too (the glob will also match my own module)
else
	for f in ../../../*Installer.app/Contents/Resources/Modules/*; do
		installModule "$f"
	done
fi


# -------------------------------------- Postflight ------------------------------------

# create preferences and write empty support files
status "Running postflight"
"$dir/FarSuiteMaster" InstallationManager afterInstall


# create log directory
[ ! -d $logDir ] && mkdir $logDir
chmod 777 $logDir
chown root:admin $logDir

echo "
====================================================================
                                      Installed
====================================================================
" >> $mainLog

chmod 666 $logDir/*


# We use the -w because if they've uninstalled it will have done so with -w
status "Launching daemon"
launchctl load -w "/Library/LaunchDaemons/$daemonId.plist" &

# Give the daemon a chance to get going. All this means is that it will be listening to logging from the app when the app starts,
# which is handy.
sleep 3

touch "$successFile"

exit 0   # if nothing else, can see this easily in the log
